from unittest import TestCase

from pdf2ebook.pdf import PDF


class PDFTest(TestCase):

    def test_to_text(self):
        pass

    def test_to_html(self):
        pdf = PDF(path='test/resources/alice.pdf')
        pdf.load()
        self.assertEquals(len(pdf.pages), 21)

    def test_pages_context(self):
        pdf = PDF(path='test/resources/alice.pdf')
        pdf.load()
        self.assertEquals(
            pdf.pages._data[10].next_page.idx,
            pdf.pages._data[11].idx,
        )

    def test_pages_content(self):
        pdf = PDF(path='test/resources/alice.pdf')
        pdf.load()

        self.assertTrue(
            pdf.pages._data[0].text_content.startswith(
                '1'
            )
        )
        pdf.pages._data[0].text_content[21:35]

        self.assertTrue(
            'in Wonderland' in pdf.pages._data[0].text_content
        )

    def test_to_epub(self):
        pdf = PDF(path='test/resources/alice.pdf')
        pdf.load()
        pdf.to_epub('/tmp/test.epub')



import os
import sys
import logging
from logging.handlers import RotatingFileHandler

from pdf2ebook.constants import LOG_LOCATION


FORMAT = "%(asctime)s|%(levelname)s| %(message)s"


class ColourfulFormatter(logging.Formatter):

    cyan = "\x1b[36;21m"
    grey = "\x1b[38;21m"
    yellow = "\x1b[33;21m"
    red = "\x1b[31;21m"
    bold_red = "\x1b[31;1m"
    reset = "\x1b[0m"
    format = FORMAT
    format_problematic = FORMAT + " (%(filename)s:%(lineno)d)"

    FORMATS = {
        logging.DEBUG: cyan + format + reset,
        logging.INFO: grey + format + reset,
        logging.WARNING: yellow + format_problematic + reset,
        logging.ERROR: red + format_problematic + reset,
        logging.CRITICAL: bold_red + format_problematic + reset,
    }

    def format(self, record):
        log_fmt = self.FORMATS.get(record.levelno)
        formatter = logging.Formatter(log_fmt)
        return formatter.format(record)


logger = logging.Logger("pdf2ebook", logging.DEBUG)

os.makedirs(os.path.dirname(LOG_LOCATION), exist_ok=True)

handler = RotatingFileHandler(LOG_LOCATION, maxBytes=1000000, backupCount=20)
handler.setFormatter(
    logging.Formatter(
        fmt="%(asctime)s|%(levelname)s| %(message)s", datefmt="%Y-%m-%d %H:%M:%S"
    )
)
logger.addHandler(handler)

std_handler = logging.StreamHandler(sys.stdout)
std_handler.setFormatter(ColourfulFormatter())
logger.addHandler(std_handler)



from pdf2ebook.utils import window


class Pages:
    def __init__(self, *args, **kwargs):
        self._data = []

    def append(self, item):
        self._data.append(item)

    def __len__(self):
        return len(self._data)

    def __getitem__(self, i):
        return self._data[i]

    def __iter__(self):
        return (i for i in self._data)

    def set_context(self):
        for one, two in window(self._data, window_size=2):
            one.next_page = two



import os
from io import StringIO
import shutil

import bs4
import langdetect
from cached_property import cached_property

from ebooklib import epub
from ebooklib.utils import create_pagebreak


class GenericPage:

    @cached_property
    def lang(self):
        try:
            lang = langdetect.detect(self.text_content[:1000])
        except langdetect.lang_detect_exception.LangDetectException:
            lang = 'en'

        return lang


class Page(GenericPage):
    def __init__(self, idx, content):
        self.idx = idx
        self.next_page = None
        self.content = content

    @property
    def text_content(self):
        if self.next_page is not None:
            return "\n".join(self.content.split("\x0c")[self.idx : self.next_page.idx])
        return "\n".join(self.content.split("\x0c")[self.idx :])

    @property
    def page_no(self):
        # Page number and written page number different
        # this is what's written on the page, not the actual index of the page in pages

        # TODO: find if at the top or bottom. Be told from pages when adding context
        # Also need to remove author name if first or last phrase
        first_line = self.text_content.strip().split("\n")[0]
        last_line = self.text_content.strip().split("\n")[-1]

        if first_line.isdigit():
            return first_line
        elif last_line.isdigit():
            return last_line
        else:
            return None

    @property
    def html_content(self):
        content = ""
        for para in self.text_content.split("\n\n"):
            content += "<p>" + para + "</p>"
        return content

    @property
    def epub_content(self):
        # need a different content that strips headers and footers
        c = epub.EpubHtml(
            title=f"title_{self.idx}",
            file_name=f"page_{self.idx}.xhtml",
            lang=self.lang,
            uid=str(self.idx),
        )
        c.content = self.html_content
        c.content += create_pagebreak(f"p_{self.idx}")
        return c


class HTMLPage(GenericPage):
    def __init__(self, idx, content):
        self.idx = idx
        self.next_page = None

        soup = bs4.BeautifulSoup(StringIO(content), "html.parser")
        soup.hr.decompose()

        self.content = str(soup)

    @property
    def images(self):
        soup = bs4.BeautifulSoup(StringIO(self.content), "html.parser")
        imgs = soup.find_all("img")
        images = []
        for idx, img in enumerate(imgs):
            epub_image = epub.EpubImage()

            real_path = img["src"]

            new_path = os.path.basename(real_path)
            try:
                shutil.copy(real_path, new_path)
            except:
                pass
            image_content = open(new_path, "rb").read()
            epub_image.uid = f"image_{self.idx}_{idx}"
            epub_image.file_name = new_path
            epub_image.media_type = "image/png"  # TODO: detect
            epub_image.set_content(image_content)
            images.append(epub_image)

        return images

    @property
    def text_content(self):
        soup = bs4.BeautifulSoup(StringIO(self.content), "html.parser")
        return soup.text.strip()

    @property
    def page_no(self):
        raise NotImplementedError()

    @property
    def html_content(self):
        soup = bs4.BeautifulSoup(StringIO(self.content), "html.parser")
        for img in soup.find_all("img"):
            img["src"] = os.path.basename(img["src"])
        self.content = str(soup)
        return self.content

    @property
    def epub_content(self):

        # need a different content that strips headers and footers
        epub_page = epub.EpubHtml(
            title=f"title_{self.idx}",
            file_name=f"page_{self.idx}.xhtml",
            lang=self.lang,
            uid=str(self.idx),
        )
        epub_page.content = self.html_content
        epub_page.content += create_pagebreak(f"p_{self.idx}")
        return epub_page



from setuptools import (
    find_packages,
    setup
)

INSTALL_REQUIRES = (
    'pdf2text',
    'ebooklib',
    'bs4',
    'langdetect',
    'cached_property'
)

setup(
    name='pdf2ebook',
    version='1.6.0',
    python_requires='>=3.6',
    description='PDF to ebook',
    long_description='PDF to ebook',
    author='Robert Lucey',
    url='https://github.com/RobertLucey/pdf2ebook',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    install_requires=INSTALL_REQUIRES,
    entry_points={
        'console_scripts': [
            'pdf2epub = pdf2ebook.bin.convert:main'
        ]
    }
)



import os

import bs4
from ebooklib import epub
from ebooklib.plugins import standard
from cached_property import cached_property

from pdf2ebook import logger
from pdf2ebook.page import Page, HTMLPage
from pdf2ebook.pages import Pages
from pdf2ebook.utils import window


class PDF:
    def __init__(self, *args, **kwargs):

        self.pdf_path = kwargs["path"]
        self.text_content = None

        self._use_text = kwargs.get("use_text", None)
        self._use_html = kwargs.get("use_html", None)

        self.text_file = None
        self.html_file = None

    def to_epub(self, path=None):
        if self.use_text:
            logger.warning('Only using text, images will not be included')

        book = epub.EpubBook()

        # add metadata
        book.set_title(os.path.splitext(os.path.basename(self.pdf_path))[0])

        book.add_author("")

        contents = []
        for page in self.pages:
            contents.append(page)

        if self.use_html:
            for page in contents:
                for image in page.images:
                    book.add_item(image)

        for page in contents:
            book.add_item(page.epub_content)

        book.add_item(epub.EpubNcx())
        book.add_item(epub.EpubNav())

        book.spine = ["nav"] + [c.epub_content for c in contents]

        langs = [page.lang for page in self.pages]
        lang = max(set(langs), key=langs.count)

        logger.debug(f'Language detected: {lang}')

        book.set_language(lang)

        opts = {"plugins": [standard.SyntaxPlugin()]}
        epub.write_epub(path, book, opts)
        logger.debug(f'Epub saved: {path}')

    def load_text(self):
        self.text_file = self.pdf_path + ".txt"

        if not os.path.exists(self.text_file):
            os.system(f"pdftotext '{self.pdf_path}' '{self.text_file}'")

        if not os.path.exists(self.text_file):
            logger.error("Could not convert pdf to text: %s" % (self.text_file))
            return

        self.text_content = open(self.text_file, "r").read()

    def load_html(self):
        self.html_file = self.pdf_path.replace(".pdf", "s.html")

        if not os.path.exists(self.html_file):
            os.system(f"pdftohtml '{self.pdf_path}'")

        if not os.path.exists(self.html_file):
            logger.error("Could not convert pdf to html: %s" % (self.html_file))
            self.loaded = True
            return

    def load(self):
        self.load_text()
        self.load_html()

    @property
    def use_text(self):
        if self._use_text is not None:
            return self._use_text

        if not self.use_html:
            return True

        return False

    @property
    def use_html(self):
        if self._use_html is not None:
            return self._use_html

        soup = bs4.BeautifulSoup(open(self.html_file), "html.parser")

        if len(soup.text.split(" ")) > 2000:
            return True

        return False

    @cached_property
    def pages(self):
        # TODO Find contents / table of contents and start after that. Who needs acks
        pages = Pages()

        if self.use_text:
            logger.debug('Generating pages using only text')
            # TODO: if all the content looks to be in html, use that rather than text
            for idx, (p, c, n) in enumerate(
                window(self.text_content.split("\x0c"), window_size=3)
            ):
                pages.append(Page(idx, self.text_content))

        elif self.use_html:
            logger.debug('Generating pages using html')

            soup = bs4.BeautifulSoup(open(self.html_file), "html.parser")
            html_content = open(self.html_file, "r").read()
            body = soup.find("body")
            breaks = body.find_all("hr")

            page_idx = 0

            pages.append(
                HTMLPage(
                    page_idx,
                    "\n".join(
                        html_content.split("\n")[body.sourceline : breaks[0].sourceline]
                    ),
                )
            )
            for idx, (i, j) in enumerate(window(breaks, window_size=2)):
                pages.append(
                    HTMLPage(
                        idx + 1,
                        "\n".join(
                            html_content.split("\n")[i.sourceline : j.sourceline]
                        ),
                    )
                )
        else:
            raise Exception("Could not convert")

        pages.set_context()

        logger.debug(f'Generated {len(pages)} pages')

        return pages
